@@ -44,10 +44,11 @@ static bool_t try_connect(int fd, const struct sockaddr *dest, socklen_t addrlen
 	return TRUE;
 }
 
-static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_family){
+static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_family, bool_t reuse_addr){
 	int err;
 	int optval = 1;
 	ortp_socket_t sock=-1;
+
 #ifdef ORTP_INET6
 	char num[8];
 	struct addrinfo hints, *res0, *res;
@@ -69,25 +70,26 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 	
 	for (res = res0; res; res = res->ai_next) {
 		sock = socket(res->ai_family, res->ai_socktype, 0);
-		if (sock < 0)
-		  continue;
-                
-		err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
-				  (SOCKET_OPTION_VALUE)&optval, sizeof (optval));
-		if (err < 0)
-		{
-			ortp_warning ("Fail to set rtp address reusable: %s.", getSocketError());
+		if (sock==-1)
+			continue;
+
+		if (reuse_addr){
+			err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
+					(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
+			if (err < 0)
+			{
+				ortp_warning ("Fail to set rtp address reusable: %s.", getSocketError());
+			}
 		}
 
 		*sock_family=res->ai_family;
 		err = bind (sock, res->ai_addr, res->ai_addrlen);
-		if (err != 0)
-		  {
-		    ortp_warning ("Fail to bind rtp socket to (addr=%s port=%i) : %s.", addr,port, getSocketError());
-		    close_socket (sock);
+		if (err != 0){
+			ortp_warning ("Fail to bind rtp socket to (addr=%s port=%i) : %s.", addr,port, getSocketError());
+			close_socket (sock);
 			sock=-1;
-		    continue;
-		  }
+			continue;
+		}
 #ifndef __hpux
 		switch (res->ai_family)
 		  {
@@ -98,12 +100,11 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 			  mreq.imr_multiaddr.s_addr = ((struct sockaddr_in *) res->ai_addr)->sin_addr.s_addr;
 			  mreq.imr_interface.s_addr = INADDR_ANY;
 			  err = setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (SOCKET_OPTION_VALUE) &mreq, sizeof(mreq));
-			  if (err < 0)
-			    {
-			      ortp_warning ("Fail to join address group: %s.", getSocketError());
-			      close_socket (sock);
-					sock=-1;
-			      continue;
+			  if (err < 0){
+				ortp_warning ("Fail to join address group: %s.", getSocketError());
+				close_socket (sock);
+				sock=-1;
+				continue;
 			    }
 			}
 		      break;
@@ -118,7 +119,7 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 			    {
 			      ortp_warning ("Fail to join address group: %s.", getSocketError());
 			      close_socket (sock);
-					sock=-1;
+			      sock=-1;
 			      continue;
 			    }
 			}
@@ -135,19 +136,20 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 	if (err < 0)
 	{
 		ortp_warning ("Error in socket address:%s.", getSocketError());
-		return err;
+		return -1;
 	}
 	saddr.sin_port = htons (port);
 
 	sock = socket (PF_INET, SOCK_DGRAM, 0);
-	
-	if (sock<0) return -1;
-	
-	err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
-			  (SOCKET_OPTION_VALUE)&optval, sizeof (optval));
-	if (err < 0)
-	{
-		ortp_warning ("Fail to set rtp address reusable: %s.",getSocketError());
+	if (sock==-1) return -1;
+
+	if (reuse_addr){
+		err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
+				(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
+		if (err < 0)
+		{
+			ortp_warning ("Fail to set rtp address reusable: %s.",getSocketError());
+		}
 	}
 
 	err = bind (sock,
@@ -161,26 +163,47 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 		return -1;
 	}
 #endif
-	if (sock>=0)
-	{
-#ifdef WIN32
-		/* increase RTP buffer on windows */
-		int bufsize = 32768;
-		err = setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (void *)&bufsize, sizeof(bufsize)); 
-		if (err == -1) {
-			ortp_warning ("Fail to increase buffer size for socket (port %i): %s.", port, getSocketError());
-		}
-		bufsize = 32768;
-		err = setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (void *)&bufsize, sizeof(bufsize)); 
-		if (err == -1) {
-			ortp_warning ("Fail to increase buffer size for socket (port %i): %s.", port, getSocketError());
-		}
-#endif
+	if (sock!=-1){
 		set_non_blocking_socket (sock);
 	}
 	return sock;
 }
 
+static void set_socket_sizes(int sock, unsigned int sndbufsz, unsigned int rcvbufsz){
+	int err;
+	bool_t done=FALSE;
+	if (sndbufsz>0){
+#ifdef SO_SNDBUFFORCE
+		err = setsockopt(sock, SOL_SOCKET, SO_SNDBUFFORCE, (void *)&sndbufsz, sizeof(sndbufsz)); 
+		if (err == -1) {
+			ortp_error("Fail to increase socket's send buffer size with SO_SNDBUFFORCE: %s.", getSocketError());
+		}else done=TRUE;
+#endif
+		if (!done){
+			err = setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (void *)&sndbufsz, sizeof(sndbufsz)); 
+			if (err == -1) {
+				ortp_error("Fail to increase socket's send buffer size with SO_SNDBUF: %s.", getSocketError());
+			}
+		}
+	}
+	done=FALSE;
+	if (rcvbufsz>0){
+#ifdef SO_RCVBUFFORCE
+		err = setsockopt(sock, SOL_SOCKET, SO_RCVBUFFORCE, (void *)&rcvbufsz, sizeof(rcvbufsz)); 
+		if (err == -1) {
+			ortp_error("Fail to increase socket's recv buffer size with SO_RCVBUFFORCE: %s.", getSocketError());
+		}
+#endif
+		if (!done){
+			err = setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (void *)&rcvbufsz, sizeof(rcvbufsz)); 
+			if (err == -1) {
+				ortp_error("Fail to increase socket's recv buffer size with SO_RCVBUF: %s.", getSocketError());
+			}
+		}
+		
+	}
+}
+
 static ortp_socket_t create_and_bind_random(const char *localip, int *sock_family, int *port){
 	int retry;
 	ortp_socket_t sock = -1;
@@ -192,8 +215,9 @@ static ortp_socket_t create_and_bind_random(const char *localip, int *sock_famil
 			localport = (rand () + 5000) & 0xfffe;
 		}
 		while ((localport < 5000) || (localport > 0xffff));
-		sock = create_and_bind(localip, localport, sock_family);
-		if (sock>=0) {
+		/*do not set REUSEADDR in case of random allocation */
+		sock = create_and_bind(localip, localport, sock_family,FALSE);
+		if (sock!=-1) {
 			*port=localport;
 			return sock;
 		}
@@ -222,22 +246,29 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int port)
 {
 	ortp_socket_t sock;
 	int sockfamily;
+	bool_t reuse_addr;
 	if (session->rtp.socket>=0){
 		/* don't rebind, but close before*/
 		rtp_session_release_sockets(session);
 	}
+#ifdef SO_REUSE_ADDR
+	reuse_addr=TRUE;
+#else
+	reuse_addr=FALSE;
+#endif
 	/* try to bind the rtp port */
 	if (port>0)
-		sock=create_and_bind(addr,port,&sockfamily);
+		sock=create_and_bind(addr,port,&sockfamily,reuse_addr);
 	else
 		sock=create_and_bind_random(addr,&sockfamily,&port);
-	if (sock>=0){
+	if (sock!=-1){
+		set_socket_sizes(sock,session->rtp.snd_socket_size,session->rtp.rcv_socket_size);
 		session->rtp.sockfamily=sockfamily;
 		session->rtp.socket=sock;
 		session->rtp.loc_port=port;
 		/*try to bind rtcp port */
-		sock=create_and_bind(addr,port+1,&sockfamily);
-		if (sock>=0){
+		sock=create_and_bind(addr,port+1,&sockfamily,reuse_addr);
+		if (sock!=-1){
 			session->rtcp.sockfamily=sockfamily;
 			session->rtcp.socket=sock;
 		}else{
@@ -504,9 +535,27 @@ static char * ortp_inet_ntoa(struct sockaddr *addr, int addrlen, char *dest, int
  *
  *	Returns: 0 on success.
 **/
+int
+rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port){
+	return rtp_session_set_remote_addr_full (session, addr, port, port+1);
+}
+
+/**
+ *rtp_session_set_remote_addr_full:
+ *@session:		a rtp session freshly created.
+ *@addr:		a local IP address in the xxx.xxx.xxx.xxx form.
+ *@rtp_port:		a local rtp port.
+ *@rtcp_port:		a local rtcp port.
+ *
+ *	Sets the remote address of the rtp session, ie the destination address where rtp packet
+ *	are sent. If the session is recv-only or duplex, it also sets the origin of incoming RTP 
+ *	packets. Rtp packets that don't come from addr:port are discarded.
+ *
+ *	Returns: 0 on success.
+**/
 
 int
-rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
+rtp_session_set_remote_addr_full (RtpSession * session, const char * addr, int rtp_port, int rtcp_port)
 {
 	int err;
 #ifdef ORTP_INET6
@@ -515,7 +564,7 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = PF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM;
-	snprintf(num, sizeof(num), "%d", port);
+	snprintf(num, sizeof(num), "%d", rtp_port);
 	err = getaddrinfo(addr, num, &hints, &res0);
 	if (err) {
 		ortp_warning ("Error in socket address: %s", gai_strerror(err));
@@ -549,14 +598,14 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
 	}
 	freeaddrinfo(res0);
 	if (err) {
-		ortp_warning("Could not set destination for RTP socket to %s:%i.",addr,port);
+		ortp_warning("Could not set destination for RTP socket to %s:%i.",addr,rtp_port);
 		return -1;
 	}
 	
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = PF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM;
-	snprintf(num, sizeof(num), "%d", (port + 1));
+	snprintf(num, sizeof(num), "%d", rtcp_port);
 	err = getaddrinfo(addr, num, &hints, &res0);
 	if (err) {
 		ortp_warning ("Error: %s", gai_strerror(err));
@@ -574,7 +623,7 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
 	}
 	freeaddrinfo(res0);
 	if (err) {
-		ortp_warning("Could not set destination for RCTP socket to %s:%i.",addr,port+1);
+		ortp_warning("Could not set destination for RCTP socket to %s:%i.",addr,rtcp_port);
 		return -1;
 	}
 #else
@@ -587,12 +636,12 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
 		ortp_warning ("Error in socket address:%s.", getSocketError());
 		return err;
 	}
-	session->rtp.rem_addr.sin_port = htons (port);
+	session->rtp.rem_addr.sin_port = htons (rtp_port);
 
 	memcpy (&session->rtcp.rem_addr, &session->rtp.rem_addr,
 		sizeof (struct sockaddr_in));
-	session->rtcp.rem_addr.sin_port = htons (port + 1);
-	session->rtcp.rem_addrlen=sizeof(session->rtp.rem_addr);
+	session->rtcp.rem_addr.sin_port = htons (rtcp_port);
+	session->rtcp.rem_addrlen=sizeof(session->rtcp.rem_addr);
 #endif
 	if (can_connect(session)){
 		if (try_connect(session->rtp.socket,(struct sockaddr*)&session->rtp.rem_addr,session->rtp.rem_addrlen))
@@ -618,6 +667,11 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
 	return 0;
 }
 
+int
+rtp_session_set_remote_addr_and_port(RtpSession * session, const char * addr, int rtp_port, int rtcp_port){
+	return rtp_session_set_remote_addr_full(session,addr,rtp_port,rtcp_port);
+}
+
 void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd)
 {
 	if (rtpfd>=0) set_non_blocking_socket(rtpfd);
@@ -629,6 +683,22 @@ void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd)
 	else session->flags&=~(RTP_SESSION_USING_EXT_SOCKETS|RTP_SOCKET_CONNECTED|RTCP_SOCKET_CONNECTED);
 }
 
+void rtp_session_set_transports(RtpSession *session, struct _RtpTransport *rtptr, struct _RtpTransport *rtcptr)
+{
+	session->rtp.tr = rtptr;
+	session->rtcp.tr = rtcptr;
+	if (rtptr)
+		rtptr->session=session;
+	if (rtcptr)
+		rtcptr->session=session;
+
+	if (rtptr || rtcptr )
+		session->flags|=(RTP_SESSION_USING_TRANSPORT);
+	else session->flags&=~(RTP_SESSION_USING_TRANSPORT);
+}
+
+
+
 /**
  *rtp_session_flush_sockets:
  *@session: a rtp session
@@ -640,18 +710,30 @@ void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd)
  *
 **/
 void rtp_session_flush_sockets(RtpSession *session){
-	char trash[4096];
+	unsigned char trash[4096];
 #ifdef ORTP_INET6
 	struct sockaddr_storage from;
 #else
 	struct sockaddr from;
 #endif
 	socklen_t fromlen=sizeof(from);
+	if (rtp_session_using_transport(session, rtp))
+	  {
+		mblk_t *trashmp=esballoc(trash,sizeof(trash),0,NULL);
+		
+	    while (session->rtp.tr->t_recvfrom(session->rtp.tr,trashmp,0,(struct sockaddr *)&from,&fromlen)>0){};
+
+	    if (session->rtcp.tr)
+	      while (session->rtcp.tr->t_recvfrom(session->rtcp.tr,trashmp,0,(struct sockaddr *)&from,&fromlen)>0){};
+		freemsg(trashmp);
+	    return;
+	  }
+
 	if (session->rtp.socket>=0){
-		while (recvfrom(session->rtp.socket,trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
+		while (recvfrom(session->rtp.socket,(char*)trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
 	}
 	if (session->rtcp.socket>=0){
-		while (recvfrom(session->rtcp.socket,trash,sizeof(trash),0,(struct sockaddr*)&from,&fromlen)>0){};
+		while (recvfrom(session->rtcp.socket,(char*)trash,sizeof(trash),0,(struct sockaddr*)&from,&fromlen)>0){};
 	}
 }
 
@@ -667,6 +749,9 @@ static int rtp_sendmsg(int sock,mblk_t *m, struct sockaddr *rem_addr, int addr_l
 		iov[iovlen].iov_base=m->b_rptr;
 		iov[iovlen].iov_len=m->b_wptr-m->b_rptr;
 	}
+	if (iovlen==MAX_IOV){
+		ortp_error("Too long msgb, didn't fit into iov, end discarded.");
+	}
 	msg.msg_name=(void*)rem_addr;
 	msg.msg_namelen=addr_len;
 	msg.msg_iov=&iov[0];
@@ -717,19 +802,24 @@ rtp_session_rtp_send (RtpSession * session, mblk_t * m)
 		destaddr=NULL;
 		destlen=0;
 	}
+
+	if (rtp_session_using_transport(session, rtp)){
+		error = (session->rtp.tr->t_sendto) (session->rtp.tr,m,0,destaddr,destlen);
+	}else{
 #ifdef USE_SENDMSG
-	error=rtp_sendmsg(sockfd,m,destaddr,destlen);
+		error=rtp_sendmsg(sockfd,m,destaddr,destlen);
 #else
-	if (m->b_cont!=NULL){
-		msgpullup(m,-1);
-	}
-	error = sendto (sockfd, m->b_rptr, (int) (m->b_wptr - m->b_rptr),
+		if (m->b_cont!=NULL)
+			msgpullup(m,-1);
+		error = sendto (sockfd, (char*)m->b_rptr, (int) (m->b_wptr - m->b_rptr),
 			 0,destaddr,destlen);
 #endif
+	}
 	if (error < 0){
 		if (session->on_network_error.count>0){
 			rtp_signal_table_emit3(&session->on_network_error,(long)"Error sending RTP packet",INT_TO_POINTER(getSocketErrorCode()));
 		}else ortp_warning ("Error sending rtp packet: %s ; socket=%i", getSocketError(), sockfd);
+		session->rtp.send_errno=getSocketErrorCode();
 	}else{
 		update_sent_bytes(session,error);
 	}
@@ -751,25 +841,32 @@ rtp_session_rtcp_send (RtpSession * session, mblk_t * m)
 		destlen=0;
 	}
 
-	if (sockfd>=0 && (session->rtcp.rem_addrlen>0 ||using_connected_socket) ){
-		
-#ifndef USE_SENDMSG
-		if (m->b_cont!=NULL){
-			msgpullup(m,-1);
+	if (session->rtcp.enabled &&
+		( (sockfd>=0 && (session->rtcp.rem_addrlen>0 ||using_connected_socket))
+			|| rtp_session_using_transport(session, rtcp) ) ){
+		if (rtp_session_using_transport(session, rtcp)){
+			error = (session->rtcp.tr->t_sendto) (session->rtcp.tr, m, 0,
+			destaddr, destlen);
 		}
-		error = sendto (sockfd, m->b_rptr,
+		else{
+#ifdef USE_SENDMSG
+			error=rtp_sendmsg(sockfd,m,destaddr, destlen);
+#else
+			if (m->b_cont!=NULL){
+				msgpullup(m,-1);
+			}
+			error = sendto (sockfd, (char*)m->b_rptr,
 			(int) (m->b_wptr - m->b_rptr), 0,
 			destaddr, destlen);
-#else
-		error=rtp_sendmsg(sockfd,m,destaddr, destlen);
 #endif
+		}
 		if (error < 0){
-			char host[50];
+			char host[65];
 			if (session->on_network_error.count>0){
 				rtp_signal_table_emit3(&session->on_network_error,(long)"Error sending RTCP packet",INT_TO_POINTER(getSocketErrorCode()));
 			}else ortp_warning ("Error sending rtcp packet: %s ; socket=%i; addr=%s", getSocketError(), session->rtcp.socket, ortp_inet_ntoa((struct sockaddr*)&session->rtcp.rem_addr,session->rtcp.rem_addrlen,host,sizeof(host)) );
 		}
-	}else ortp_warning("Cannot send rtcp report because I don't know the remote address.");
+	}else ortp_message("Not sending rtcp report: sockfd=%i, rem_addrlen=%i, connected=%i",sockfd,session->rtcp.rem_addrlen,using_connected_socket);
 	freemsg (m);
 	return error;
 }
@@ -786,36 +883,41 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 #endif
 	socklen_t addrlen = sizeof (remaddr);
 	mblk_t *mp;
-	bool_t sock_connected=(session->flags & RTP_SOCKET_CONNECTED);
-
-	if (sockfd<0) return -1;  /*session has no sockets for the moment*/
+	
+	if ((sockfd<0) && !rtp_session_using_transport(session, rtp)) return -1;  /*session has no sockets for the moment*/
 
 	while (1)
 	{
 		int bufsz;
+		bool_t sock_connected=!!(session->flags & RTP_SOCKET_CONNECTED);
+
 		if (session->rtp.cached_mp==NULL)
-			 session->rtp.cached_mp = allocb (session->recv_buf_size, 0);
+			 session->rtp.cached_mp = msgb_allocator_alloc(&session->allocator,session->recv_buf_size);
 		mp=session->rtp.cached_mp;
 		bufsz=(int) (mp->b_datap->db_lim - mp->b_datap->db_base);
 		if (sock_connected){
-			error=recv(sockfd,mp->b_wptr,bufsz,0);
-		}else error = recvfrom(sockfd, mp->b_wptr,
+			error=recv(sockfd,(char*)mp->b_wptr,bufsz,0);
+		}else if (rtp_session_using_transport(session, rtp)) 
+			error = (session->rtp.tr->t_recvfrom)(session->rtp.tr, mp, 0,
+				  (struct sockaddr *) &remaddr,
+				  &addrlen);
+		else error = recvfrom(sockfd, (char*)mp->b_wptr,
 				  bufsz, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
 		if (error > 0){
 			if (session->symmetric_rtp && !sock_connected){
-				/* store the sender rtp address to do symmetric RTP */
-				memcpy(&session->rtp.rem_addr,&remaddr,addrlen);
-				session->rtp.rem_addrlen=addrlen;
 				if (session->use_connect){
+					/* store the sender rtp address to do symmetric RTP */
+					memcpy(&session->rtp.rem_addr,&remaddr,addrlen);
+					session->rtp.rem_addrlen=addrlen;
 					if (try_connect(sockfd,(struct sockaddr*)&remaddr,addrlen))
 						session->flags|=RTP_SOCKET_CONNECTED;
 				}
 			}
 			/* then parse the message and put on queue */
 			mp->b_wptr+=error;
-			rtp_session_rtp_parse (session, mp, user_ts + session->rtp.hwrcv_diff_ts, (struct sockaddr*)&remaddr,addrlen);
+			rtp_session_rtp_parse (session, mp, user_ts, (struct sockaddr*)&remaddr,addrlen);
 			session->rtp.cached_mp=NULL;
 			/*for bandwidth measurements:*/
 			update_recv_bytes(session,error);
@@ -849,6 +951,7 @@ void rtp_session_notify_inc_rtcp(RtpSession *session, mblk_t *m){
 		d->packet=m;
 		rtp_session_dispatch_event(session,ev);
 	}
+	else freemsg(m);  /* avoid memory leak */
 }
 
 int
@@ -862,22 +965,28 @@ rtp_session_rtcp_recv (RtpSession * session)
 #endif
 	socklen_t addrlen=0;
 	mblk_t *mp;
-	bool_t sock_connected=session->flags & RTCP_SOCKET_CONNECTED;
 
-	if (session->rtcp.socket<0) return -1;  /*session has no rtcp sockets for the moment*/
+	if (session->rtcp.socket<0 && !rtp_session_using_transport(session, rtcp)) return -1;  /*session has no rtcp sockets for the moment*/
 	
 
 	while (1)
 	{
+		bool_t sock_connected=!!(session->flags & RTCP_SOCKET_CONNECTED);
 		if (session->rtcp.cached_mp==NULL)
 			 session->rtcp.cached_mp = allocb (RTCP_MAX_RECV_BUFSIZE, 0);
 		
 		mp=session->rtcp.cached_mp;
 		if (sock_connected){
-			error=recv(session->rtcp.socket,mp->b_wptr,RTCP_MAX_RECV_BUFSIZE,0);
+			error=recv(session->rtcp.socket,(char*)mp->b_wptr,RTCP_MAX_RECV_BUFSIZE,0);
 		}else {
 			addrlen=sizeof (remaddr);
-			error=recvfrom (session->rtcp.socket, mp->b_wptr,
+
+			if (rtp_session_using_transport(session, rtcp))
+			  error=(session->rtcp.tr->t_recvfrom)(session->rtcp.tr, mp, 0,
+				  (struct sockaddr *) &remaddr,
+				  &addrlen);
+			else
+			  error=recvfrom (session->rtcp.socket,(char*) mp->b_wptr,
 				  RTCP_MAX_RECV_BUFSIZE, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
@@ -914,6 +1023,7 @@ rtp_session_rtcp_recv (RtpSession * session)
 				if (session->on_network_error.count>0){
 					rtp_signal_table_emit3(&session->on_network_error,(long)"Error receiving RTCP packet",INT_TO_POINTER(errnum));
 				}else ortp_warning("Error receiving RTCP packet: %s.",getSocketError());
+				session->rtp.recv_errno=errnum;
 			}
 			/* don't free the cached_mp, it will be reused next time */
 			return -1;	/* avoids an infinite loop ! */
